/*
	ProWesS, PROGS Windowing System

	Colour Select type "CSEL"

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 25 November 2000
*/

#include <mem.h>
#include <dir.h>
#include <str.h>
#include <dev.h>
#include <ctype.h>

#include "PWHandler.h"

#define TYPE PW_TYPE_COLOURSELECT
#define catch(X) do { if (err=(X)) return err; } while (0)

/* loose item object definition */

typedef struct {
	PWBaseObject;
	PWObject outl, list, colbox, compbox;
	PWObject butgray, butrgb, butcmyk;
	PWObject comp1, comp2, comp3, comp4;
	int colourtype;
	ColourRGB rgb;
	ColourCMYK cmyk;
	pt gray;
} Object;

/* object handling routines */

Error Create(PWObject owner, PWObject *ret, PWEvent event, int *taglist)
{
	/* allocate specific object - which should be cleared (automatic) */
	return PWAllocate(event.region->system,sizeof(Object),ret);
}

Error Remove(PWObject object)
{
	Object *obj=(Object *)object;

	if (obj->outl)
		PWRemove(obj->outl);
	return PWRelease((PWObject)obj,(PWObject)obj);
}

Error Handle(PWObject object, PWWait event)
{
	Object *obj=(Object *)object;
	char dir[IO_MAXDIRECTORY];
	Error err;

	PWChange(obj->outl,PW_WINDOW_ORIGIN_RESET,NULL);
	err=PWActivate(obj->outl);
	return err;
}

Error exit(PWObject object)
{
	return PWChange(object,PW_SYSTEM_BREAKDOWN,NULL);
}

Error redrawPreview(PWObject object, CanvasInfo *rinfo)
{
	int i,j,siz;
	Gstate gid=rinfo->gstate;
	Object *obj;

	PWQuery(object,PW_GLOBAL_AUXILIARY,&obj);

	/* adjust bbox to fit canvas */
	switch (obj->colourtype)
	{
	case PW_COLOURSELECT_GRAY:
		PFPaperColourGray(gid,obj->gray);
		break;
	case PW_COLOURSELECT_RGB:
		PFPaperColourRGB(gid,obj->rgb);
		break;
	case PW_COLOURSELECT_CMYK:
		PFPaperColourCMYK(gid,obj->cmyk);
		break;
	}
	return PFPageClear(gid);	/* clear background */
}

Error gray(PWObject object) {
	Object *obj;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&obj);

	PWChange(object, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);

	if (obj->type!=PW_COLOURSELECT_GRAY) {
		PWChange(obj->butcmyk, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		PWChange(obj->butrgb, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		/* ..... */
	}
	return ERR_OK;
}
Error rgb(PWObject object) {
	Object *obj;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&obj);

	PWChange(object, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);

	if (obj->type!=PW_COLOURSELECT_GRAY) {
		PWChange(obj->butcmyk, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		PWChange(obj->butgray, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		/* ..... */
	}
	return ERR_OK;
}
Error cmyk(PWObject object) {
	Object *obj;
	PWQuery(object,PW_GLOBAL_AUXILIARY,&obj);

	PWChange(object, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);

	if (obj->type!=PW_COLOURSELECT_GRAY) {
		PWChange(obj->butgray, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		PWChange(obj->butrgb, PW_LOOSE_CHANGE_STATUS, PW_STATUS_SELECTED);
		/* ..... */
	}
	return ERR_OK;
}
Error preview(PWObject object)
{
	/* values have changed, display the preview */
	/* ..... */
}

Error Change(PWObject object, int *taglist)
{
	Object *obj=(Object *)object;
	PWSystem *system=obj->event.region->system;
	Error err=ERR_OK;
	char unrecognised_tag=FALSE;

	if (!(obj->status&OSTATUS_INITIALISED))
	{
		PWObject box;
		List *dev;

		obj->status|=OSTATUS_INITIALISED;

		/* create all the objects for the directory select window */
		catch( PWCreate(NULL,&obj->outl,PW_TYPE_OUTLINE,
						PW_OUTLINE_QUIT,
						PW_OUTLINE_QUIT_KEYPRESS, 27,
						PW_GLOBAL_AUXILIARY, obj,
						PW_OUTLINE_TITLE_TEXT, "colour select",
						NULL) );

		/* create preview objects */
		catch( PWCreate(obj->outl,&box,PW_TYPE_DIRECTION,
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(box,NULL,PW_TYPE_LABEL,
						PW_LABEL_TEXT, "preview",
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(box,&obj->preview,PW_TYPE_CANVAS,
						PW_CANVAS_ACTION_REDRAW, &redrawPreview,
						PW_CANVAS_POINTER, SPRITE_HAND,
						NULL) );

		/* create colour type selection objects, with label */
		catch( PWCreate(obj->outl,&box,PW_TYPE_DIRECTION,
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(box,&obj->butgray,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_ACTION_HIT, &gray,
						PW_LOOSE_TEXT, "Gray",
						PW_LOOSE_KEYPRESS, 'g',
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(box,&obj->butrgb,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_ACTION_HIT, &rgb,
						PW_LOOSE_TEXT, "RGB",
						PW_LOOSE_KEYPRESS, 'r',
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(box,&obj->butcmyk,PW_TYPE_LOOSE_ITEM,
						PW_LOOSE_ACTION_HIT, &cmyk,
						PW_LOOSE_TEXT, "CMYK",
						PW_LOOSE_KEYPRESS, 'c',
						PW_SCALE_FACTOR, 0,
						NULL) );

		/* colour components */
		catch( PWCreate(obj->outl,&obj->colbox,PW_TYPE_DIRECTION,
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(obj->colbox,&obj->compbox,PW_TYPE_DIRECTION,
						PW_SCALE_FACTOR, 0,
						NULL) );
		catch( PWCreate(obj->compbox,&obj->comp1,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_KEYPRESS, '1',
						PW_EDLINE_ACTION_AFTER, &preview,
						PW_EDLINE_WIDTH_FS, short2pt(12),
						NULL) );
		catch( PWCreate(obj->compbox,&obj->comp2,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_KEYPRESS, '2',
						PW_EDLINE_ACTION_AFTER, &preview,
						PW_EDLINE_WIDTH_FS, short2pt(12),
						NULL) );
		catch( PWCreate(obj->compbox,&obj->comp3,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_KEYPRESS, '3',
						PW_EDLINE_ACTION_AFTER, &preview,
						PW_EDLINE_WIDTH_FS, short2pt(12),
						NULL) );
		catch( PWCreate(obj->compbox,&obj->comp3,PW_TYPE_EDLINE,
						PW_EDLINE_MAXLENGTH, 10,
						PW_EDLINE_KEYPRESS, '3',
						PW_EDLINE_ACTION_AFTER, &preview,
						PW_EDLINE_WIDTH_FS, short2pt(12),
						NULL) );
	}

	while (*taglist)
	{
		switch (*taglist)
		{
		case PW_DIRSELECT_SET:
			PWChange(obj->dir,PW_EDLINE_SET,(char *)(*(taglist+1)),NULL);
			break;
		case PW_DIRSELECT_TITLE_TEXT:
			PWChange(obj->outl,PW_OUTLINE_TITLE_TEXT,(char *)(*(taglist+1)),NULL);
			break;
		case PW_DIRSELECT_ACTIVATE:
			Handle(object,0);
			break;
		case PW_DIRSELECT_ACTION:
			obj->After=(Error (*)(PWObject,char *))(*(taglist+1));
			break;
		case PW_DIRSELECT_GET:
			PWChange(obj->dir,PW_EDLINE_GET,(*(taglist+1)),(*(taglist+2)),NULL);
			break;
		default :
			if (*taglist&(~PW_TAGMASK_SKIP))
				unrecognised_tag=TRUE;
			break;
		}
		taglist=PWSkipTag(taglist);
	}
	if (unrecognised_tag) err=ERR_IPAR;
	return err;
}

Error Define(char *name, char *value)
{
	List *link, *new;
	Error err;

	if (STRSameCD(name,"DIRSELECT-DEVICE"))
	{
		link=&devices;
linkname:
		catch( MEMAllocate(sizeof(List)-1+strlen(value),&new) );
		STRCopy(value,new->name);
		while (link->next) link=link->next;
		link->next=new;
		return ERR_OK;
	}
	if (STRSameCD(name,"DIRSELECT-DIRECTORY"))
	{
		link=&directories;
		goto linkname;
	}
	return ERR_IPAR;
}

PWTypeDef init={
	PW_TYPEDEF_FLAG, TYPE, PW_TYPE_EVENT_KEYPRESS,
	&Create, &Change, NULL, &Remove, NULL, NULL, &Handle, &Define,
	NULL, NULL, NULL
};
